@@ -1,10 +1,11 @@
 /*
- * General purpose random utilities
+ * SPDX-License-Identifier: BSD-3-Clause
  *
- * Based on libuuid code.
+ * General purpose random utilities. Based on libuuid code.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * This code is free software; you can redistribute it and/or modify it under
+ * the terms of the Modified BSD License. The complete text of the license is
+ * available in the Documentation/licenses/COPYING.BSD-3-Clause file.
  */
 #include <stdio.h>
 #include <unistd.h>
@@ -36,6 +37,8 @@
 
 #if !defined(HAVE_GETRANDOM) && defined(SYS_getrandom)
 /* libc without function, but we have syscal */
+#define GRND_NONBLOCK 0x01
+#define GRND_RANDOM 0x02
 static int getrandom(void *buf, size_t buflen, unsigned int flags)
 {
 	return (syscall(SYS_getrandom, buf, buflen, flags));
@@ -57,9 +60,12 @@ static void crank_random(void)
 {
 	int i;
 	struct timeval tv;
+	unsigned int n_pid, n_uid;
 
 	gettimeofday(&tv, NULL);
-	srand((getpid() << 16) ^ getuid() ^ tv.tv_sec ^ tv.tv_usec);
+	n_pid = getpid();
+	n_uid = getuid();
+	srand((n_pid << 16) ^ n_uid ^ tv.tv_sec ^ tv.tv_usec);
 
 #ifdef DO_JRAND_MIX
 	ul_jrand_seed[0] = getpid() ^ (tv.tv_sec & 0xFFFF);
@@ -93,6 +99,9 @@ int random_get_fd(void)
  * Use /dev/urandom if possible, and if not,
  * use glibc pseudo-random functions.
  */
+#define UL_RAND_READ_ATTEMPTS	8
+#define UL_RAND_READ_DELAY	125000	/* microseconds */
+
 void random_get_bytes(void *buf, size_t nbytes)
 {
 	unsigned char *cp = (unsigned char *)buf;
@@ -104,14 +113,19 @@ void random_get_bytes(void *buf, size_t nbytes)
 		int x;
 
 		errno = 0;
-		x = getrandom(cp, n, 0);
+		x = getrandom(cp, n, GRND_NONBLOCK);
 		if (x > 0) {			/* success */
 		       n -= x;
 		       cp += x;
 		       lose_counter = 0;
-		} else if (errno == ENOSYS)	/* kernel without getrandom() */
+
+		} else if (errno == ENOSYS) {	/* kernel without getrandom() */
 			break;
-		else if (lose_counter++ > 16)	/* entropy problem? */
+
+		} else if (errno == EAGAIN && lose_counter < UL_RAND_READ_ATTEMPTS) {
+			xusleep(UL_RAND_READ_DELAY);	/* no etropy, wait and try again */
+			lose_counter++;
+		} else
 			break;
 	}
 
@@ -130,8 +144,9 @@ void random_get_bytes(void *buf, size_t nbytes)
 			while (n > 0) {
 				ssize_t x = read(fd, cp, n);
 				if (x <= 0) {
-					if (lose_counter++ > 16)
+					if (lose_counter++ > UL_RAND_READ_ATTEMPTS)
 						break;
+					xusleep(UL_RAND_READ_DELAY);
 					continue;
 				}
 				n -= x;
