@@ -150,141 +150,9 @@ struct gpt_legacy_mbr {
 		.name = (_n),    \
 	}
 
-/* Probably the most complete list of the GUIDs are at:
- *	 https://wikipedia.org/wiki/GUID_Partition_Table
- */
 static struct fdisk_parttype gpt_parttypes[] =
 {
-	/* Generic OS */
-	DEF_GUID("C12A7328-F81F-11D2-BA4B-00A0C93EC93B", N_("EFI System")),
-
-	DEF_GUID("024DEE41-33E7-11D3-9D69-0008C781F39F", N_("MBR partition scheme")),
-	DEF_GUID("D3BFE2DE-3DAF-11DF-BA40-E3A556D89593", N_("Intel Fast Flash")),
-
-	/* Hah!IdontneedEFI */
-	DEF_GUID("21686148-6449-6E6F-744E-656564454649", N_("BIOS boot")),
-
-	/* NIH syndrome */
-	DEF_GUID("F4019732-066E-4E12-8273-346C5641494F", N_("Sony boot partition")),
-	DEF_GUID("BFBFAFE7-A34F-448A-9A5B-6213EB736C22", N_("Lenovo boot partition")),
-
-	/* PowerPC reference platform boot partition */
-	DEF_GUID("9E1A2D38-C612-4316-AA26-8B49521E5A8B", N_("PowerPC PReP boot")),
-
-	/* Open Network Install Environment */
-	DEF_GUID("7412F7D5-A156-4B13-81DC-867174929325", N_("ONIE boot")),
-	DEF_GUID("D4E6E2CD-4469-46F3-B5CB-1BFF57AFC149", N_("ONIE config")),
-
-	/* Windows */
-	DEF_GUID("E3C9E316-0B5C-4DB8-817D-F92DF00215AE", N_("Microsoft reserved")),
-	DEF_GUID("EBD0A0A2-B9E5-4433-87C0-68B6B72699C7", N_("Microsoft basic data")),
-	DEF_GUID("5808C8AA-7E8F-42E0-85D2-E1E90434CFB3", N_("Microsoft LDM metadata")),
-	DEF_GUID("AF9B60A0-1431-4F62-BC68-3311714A69AD", N_("Microsoft LDM data")),
-	DEF_GUID("DE94BBA4-06D1-4D40-A16A-BFD50179D6AC", N_("Windows recovery environment")),
-	DEF_GUID("37AFFC90-EF7D-4E96-91C3-2D7AE055B174", N_("IBM General Parallel Fs")),
-	DEF_GUID("E75CAF8F-F680-4CEE-AFA3-B001E56EFC2D", N_("Microsoft Storage Spaces")),
-
-	/* HP-UX */
-	DEF_GUID("75894C1E-3AEB-11D3-B7C1-7B03A0000000", N_("HP-UX data")),
-	DEF_GUID("E2A1E728-32E3-11D6-A682-7B03A0000000", N_("HP-UX service")),
-
-	/* Linux (http://www.freedesktop.org/wiki/Specifications/DiscoverablePartitionsSpec) */
-	DEF_GUID("0657FD6D-A4AB-43C4-84E5-0933C84B4F4F", N_("Linux swap")),
-	DEF_GUID("0FC63DAF-8483-4772-8E79-3D69D8477DE4", N_("Linux filesystem")),
-	DEF_GUID("3B8F8425-20E0-4F3B-907F-1A25A76F98E8", N_("Linux server data")),
-	DEF_GUID("44479540-F297-41B2-9AF7-D131D5F0458A", N_("Linux root (x86)")),
-	DEF_GUID("69DAD710-2CE4-4E3C-B16C-21A1D49ABED3", N_("Linux root (ARM)")),
-	DEF_GUID("4F68BCE3-E8CD-4DB1-96E7-FBCAF984B709", N_("Linux root (x86-64)")),
-	DEF_GUID("B921B045-1DF0-41C3-AF44-4C6F280D3FAE", N_("Linux root (ARM-64)")),
-	DEF_GUID("993D8D3D-F80E-4225-855A-9DAF8ED7EA97", N_("Linux root	(IA-64)")),
-	DEF_GUID("8DA63339-0007-60C0-C436-083AC8230908", N_("Linux reserved")),
-	DEF_GUID("933AC7E1-2EB4-4F13-B844-0E14E2AEF915", N_("Linux home")),
-	DEF_GUID("A19D880F-05FC-4D3B-A006-743F0F84911E", N_("Linux RAID")),
-	DEF_GUID("BC13C2FF-59E6-4262-A352-B275FD6F7172", N_("Linux extended boot")),
-	DEF_GUID("E6D6D379-F507-44C2-A23C-238F2A3DF928", N_("Linux LVM")),
-	/* ... too crazy, ignore for now:
-	DEF_GUID("7FFEC5C9-2D00-49B7-8941-3EA10A5586B7", N_("Linux plain dm-crypt")),
-	DEF_GUID("CA7D7CCB-63ED-4C53-861C-1742536059CC", N_("Linux LUKS")),
-	*/
-
-	/* FreeBSD */
-	DEF_GUID("516E7CB4-6ECF-11D6-8FF8-00022D09712B", N_("FreeBSD data")),
-	DEF_GUID("83BD6B9D-7F41-11DC-BE0B-001560B84F0F", N_("FreeBSD boot")),
-	DEF_GUID("516E7CB5-6ECF-11D6-8FF8-00022D09712B", N_("FreeBSD swap")),
-	DEF_GUID("516E7CB6-6ECF-11D6-8FF8-00022D09712B", N_("FreeBSD UFS")),
-	DEF_GUID("516E7CBA-6ECF-11D6-8FF8-00022D09712B", N_("FreeBSD ZFS")),
-	DEF_GUID("516E7CB8-6ECF-11D6-8FF8-00022D09712B", N_("FreeBSD Vinum")),
-
-	/* Apple OSX */
-	DEF_GUID("48465300-0000-11AA-AA11-00306543ECAC", N_("Apple HFS/HFS+")),
-	DEF_GUID("55465300-0000-11AA-AA11-00306543ECAC", N_("Apple UFS")),
-	DEF_GUID("52414944-0000-11AA-AA11-00306543ECAC", N_("Apple RAID")),
-	DEF_GUID("52414944-5F4F-11AA-AA11-00306543ECAC", N_("Apple RAID offline")),
-	DEF_GUID("426F6F74-0000-11AA-AA11-00306543ECAC", N_("Apple boot")),
-	DEF_GUID("4C616265-6C00-11AA-AA11-00306543ECAC", N_("Apple label")),
-	DEF_GUID("5265636F-7665-11AA-AA11-00306543ECAC", N_("Apple TV recovery")),
-	DEF_GUID("53746F72-6167-11AA-AA11-00306543ECAC", N_("Apple Core storage")),
-
-	/* Solaris */
-	DEF_GUID("6A82CB45-1DD2-11B2-99A6-080020736631", N_("Solaris boot")),
-	DEF_GUID("6A85CF4D-1DD2-11B2-99A6-080020736631", N_("Solaris root")),
-	/* same as Apple ZFS */
-	DEF_GUID("6A898CC3-1DD2-11B2-99A6-080020736631", N_("Solaris /usr & Apple ZFS")),
-	DEF_GUID("6A87C46F-1DD2-11B2-99A6-080020736631", N_("Solaris swap")),
-	DEF_GUID("6A8B642B-1DD2-11B2-99A6-080020736631", N_("Solaris backup")),
-	DEF_GUID("6A8EF2E9-1DD2-11B2-99A6-080020736631", N_("Solaris /var")),
-	DEF_GUID("6A90BA39-1DD2-11B2-99A6-080020736631", N_("Solaris /home")),
-	DEF_GUID("6A9283A5-1DD2-11B2-99A6-080020736631", N_("Solaris alternate sector")),
-	DEF_GUID("6A945A3B-1DD2-11B2-99A6-080020736631", N_("Solaris reserved 1")),
-	DEF_GUID("6A9630D1-1DD2-11B2-99A6-080020736631", N_("Solaris reserved 2")),
-	DEF_GUID("6A980767-1DD2-11B2-99A6-080020736631", N_("Solaris reserved 3")),
-	DEF_GUID("6A96237F-1DD2-11B2-99A6-080020736631", N_("Solaris reserved 4")),
-	DEF_GUID("6A8D2AC7-1DD2-11B2-99A6-080020736631", N_("Solaris reserved 5")),
-
-	/* NetBSD */
-	DEF_GUID("49F48D32-B10E-11DC-B99B-0019D1879648", N_("NetBSD swap")),
-	DEF_GUID("49F48D5A-B10E-11DC-B99B-0019D1879648", N_("NetBSD FFS")),
-	DEF_GUID("49F48D82-B10E-11DC-B99B-0019D1879648", N_("NetBSD LFS")),
-	DEF_GUID("2DB519C4-B10E-11DC-B99B-0019D1879648", N_("NetBSD concatenated")),
-	DEF_GUID("2DB519EC-B10E-11DC-B99B-0019D1879648", N_("NetBSD encrypted")),
-	DEF_GUID("49F48DAA-B10E-11DC-B99B-0019D1879648", N_("NetBSD RAID")),
-
-	/* ChromeOS */
-	DEF_GUID("FE3A2A5D-4F32-41A7-B725-ACCC3285A309", N_("ChromeOS kernel")),
-	DEF_GUID("3CB8E202-3B7E-47DD-8A3C-7FF2A13CFCEC", N_("ChromeOS root fs")),
-	DEF_GUID("2E0A753D-9E48-43B0-8337-B15192CB1B5E", N_("ChromeOS reserved")),
-
-	/* MidnightBSD */
-	DEF_GUID("85D5E45A-237C-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD data")),
-	DEF_GUID("85D5E45E-237C-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD boot")),
-	DEF_GUID("85D5E45B-237C-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD swap")),
-	DEF_GUID("0394EF8B-237E-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD UFS")),
-	DEF_GUID("85D5E45D-237C-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD ZFS")),
-	DEF_GUID("85D5E45C-237C-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD Vinum")),
-
-	/* Ceph */
-	DEF_GUID("45B0969E-9B03-4F30-B4C6-B4B80CEFF106", N_("Ceph Journal")),
-	DEF_GUID("45B0969E-9B03-4F30-B4C6-5EC00CEFF106", N_("Ceph Encrypted Journal")),
-	DEF_GUID("4FBD7E29-9D25-41B8-AFD0-062C0CEFF05D", N_("Ceph OSD")),
-	DEF_GUID("4FBD7E29-9D25-41B8-AFD0-5EC00CEFF05D", N_("Ceph crypt OSD")),
-	DEF_GUID("89C57F98-2FE5-4DC0-89C1-F3AD0CEFF2BE", N_("Ceph disk in creation")),
-	DEF_GUID("89C57F98-2FE5-4DC0-89C1-5EC00CEFF2BE", N_("Ceph crypt disk in creation")),
-
-	/* VMware */
-	DEF_GUID("AA31E02A-400F-11DB-9590-000C2911D1B8", N_("VMware VMFS")),
-	DEF_GUID("9D275380-40AD-11DB-BF97-000C2911D1B8", N_("VMware Diagnostic")),
-	DEF_GUID("381CFCCC-7288-11E0-92EE-000C2911D0B2", N_("VMware Virtual SAN")),
-	DEF_GUID("77719A0C-A4A0-11E3-A47E-000C29745A24", N_("VMware Virsto")),
-	DEF_GUID("9198EFFC-31C0-11DB-8F78-000C2911D1B8", N_("VMware Reserved")),
-
-	/* OpenBSD */
-	DEF_GUID("824CC7A0-36A8-11E3-890A-952519AD3F61", N_("OpenBSD data")),
-
-	/* QNX */
-	DEF_GUID("CEF5A9AD-73BC-4601-89F3-CDEEEEE321A1", N_("QNX6 file system")),
-
-	/* Plan 9 */
-	DEF_GUID("C91818F9-8025-47AF-89D2-F030D7000C2C", N_("Plan 9 partition"))
+	#include "pt-gpt-partnames.h"
 };
 
 #define alignment_required(_x)  ((_x)->grain != (_x)->sector_size)
@@ -503,7 +371,7 @@ static inline int gpt_calculate_sizeof_ents(struct gpt_header *hdr, uint32_t nen
 		return -ERANGE;
 	}
 
-	*sz = nents * esz;
+	*sz = (size_t) nents * esz;
 	return 0;
 }
 
@@ -605,7 +473,7 @@ static void gpt_mknew_header_common(struct fdisk_context *cxt,
 		header->alternative_lba = cpu_to_le64(cxt->total_sectors - 1ULL);
 		header->partition_entry_lba = cpu_to_le64(2ULL);
 	} else { /* backup */
-		uint64_t esz = (uint64_t) le32_to_cpu(header->npartition_entries) 
+		uint64_t esz = (uint64_t) le32_to_cpu(header->npartition_entries)
 							* sizeof(struct gpt_entry);
 		uint64_t esects = (esz + cxt->sector_size - 1) / cxt->sector_size;
 
@@ -865,17 +733,20 @@ static int valid_pmbr(struct fdisk_context *cxt)
 	if (ret != GPT_MBR_PROTECTIVE)
 		goto done;
 
+
+	for (i = 0 ; i < 4; i++) {
+		if ((pmbr->partition_record[i].os_type != EFI_PMBR_OSTYPE) &&
+		    (pmbr->partition_record[i].os_type != 0x00)) {
+			ret = GPT_MBR_HYBRID;
+			goto done;
+		}
+	}
+
 	/* LBA of the GPT partition header */
 	if (pmbr->partition_record[part].starting_lba !=
 	    cpu_to_le32(GPT_PRIMARY_PARTITION_TABLE_LBA))
 		goto done;
 
-	for (i = 0 ; i < 4; i++) {
-		if ((pmbr->partition_record[i].os_type != EFI_PMBR_OSTYPE) &&
-		    (pmbr->partition_record[i].os_type != 0x00))
-			ret = GPT_MBR_HYBRID;
-	}
-
 	/*
 	 * Protective MBRs take up the lesser of the whole disk
 	 * or 2 TiB (32bit LBA), ignoring the rest of the disk.
@@ -896,7 +767,7 @@ static int valid_pmbr(struct fdisk_context *cxt)
 					   "will be corrected by write."),
 					sz_lba, cxt->total_sectors - 1ULL);
 
-			/* Note that gpt_write_pmbr() overwrites PMBR, but we want to keep it valid already 
+			/* Note that gpt_write_pmbr() overwrites PMBR, but we want to keep it valid already
 			 * in memory too to disable warnings when valid_pmbr() called next time */
 			pmbr->partition_record[part].size_in_lba  =
 				cpu_to_le32((uint32_t) min( cxt->total_sectors - 1ULL, 0xFFFFFFFFULL) );
@@ -904,6 +775,9 @@ static int valid_pmbr(struct fdisk_context *cxt)
 		}
 	}
 done:
+	DBG(LABEL, ul_debug("PMBR type: %s",
+			ret == GPT_MBR_PROTECTIVE ? "protective" :
+			ret == GPT_MBR_HYBRID     ? "hybrid"     : "???" ));
 	return ret;
 }
 
@@ -945,7 +819,7 @@ static ssize_t read_lba(struct fdisk_context *cxt, uint64_t lba,
 static unsigned char *gpt_read_entries(struct fdisk_context *cxt,
 					 struct gpt_header *header)
 {
-	size_t sz;
+	size_t sz = 0;
 	ssize_t ssz;
 
 	unsigned char *ret = NULL;
@@ -1571,9 +1445,6 @@ static int gpt_probe_label(struct fdisk_context *cxt)
 	if (!mbr_type)
 		goto failed;
 
-	DBG(LABEL, ul_debug("found a %s MBR", mbr_type == GPT_MBR_PROTECTIVE ?
-			    "protective" : "hybrid"));
-
 	/* primary header */
 	gpt->pheader = gpt_read_header(cxt, GPT_PRIMARY_PARTITION_TABLE_LBA,
 				       &gpt->ents);
@@ -1971,7 +1842,7 @@ static int gpt_write_partitions(struct fdisk_context *cxt,
 				struct gpt_header *header, unsigned char *ents)
 {
 	off_t offset = (off_t) le64_to_cpu(header->partition_entry_lba) * cxt->sector_size;
-	size_t towrite;
+	size_t towrite = 0;
 	ssize_t ssz;
 	int rc;
 
@@ -2023,6 +1894,7 @@ static int gpt_write_pmbr(struct fdisk_context *cxt)
 	assert(cxt);
 	assert(cxt->firstsector);
 
+	DBG(LABEL, ul_debug("(over)writing PMBR"));
 	pmbr = (struct gpt_legacy_mbr *) cxt->firstsector;
 
 	/* zero out the legacy partitions */
@@ -2111,8 +1983,7 @@ static int gpt_write_disklabel(struct fdisk_context *cxt)
 		goto err1;
 
 	if (mbr_type == GPT_MBR_HYBRID)
-		fdisk_warnx(cxt, _("The device contains hybrid MBR -- writing GPT only. "
-				   "You have to sync the MBR manually."));
+		fdisk_warnx(cxt, _("The device contains hybrid MBR -- writing GPT only."));
 	else if (gpt_write_pmbr(cxt) != 0)
 		goto err1;
 
@@ -2444,13 +2315,14 @@ static int gpt_add_partition(
 			if (!ask)
 				return -ENOMEM;
 
-			fdisk_ask_set_query(ask, _("Last sector, +sectors or +size{K,M,G,T,P}"));
+			fdisk_ask_set_query(ask, _("Last sector, +/-sectors or +/-size{K,M,G,T,P}"));
 			fdisk_ask_set_type(ask, FDISK_ASKTYPE_OFFSET);
 			fdisk_ask_number_set_low(ask,     user_f);	/* minimal */
 			fdisk_ask_number_set_default(ask, dflt_l);	/* default */
 			fdisk_ask_number_set_high(ask,    dflt_l);	/* maximal */
 			fdisk_ask_number_set_base(ask,    user_f);	/* base for relative input */
 			fdisk_ask_number_set_unit(ask,    cxt->sector_size);
+			fdisk_ask_number_set_wrap_negative(ask, 1);	/* wrap negative around high */
 
 			rc = fdisk_do_ask(cxt, ask);
 			if (rc)
@@ -2743,8 +2615,13 @@ int fdisk_gpt_set_npartitions(struct fdisk_context *cxt, uint32_t entries)
 	/* calculate the size (bytes) of the entries array */
 	rc = gpt_calculate_sizeof_ents(gpt->pheader, entries, &new_size);
 	if (rc) {
-		fdisk_warnx(cxt, _("The number of the partition has to be smaller than %zu."),
-				UINT32_MAX / le32_to_cpu(gpt->pheader->sizeof_partition_entry));
+		uint32_t entry_size = le32_to_cpu(gpt->pheader->sizeof_partition_entry);
+
+		if (entry_size == 0)
+			fdisk_warnx(cxt, _("The partition entry size is zero."));
+		else
+			fdisk_warnx(cxt, _("The number of the partition has to be smaller than %zu."),
+				UINT32_MAX / entry_size);
 		return rc;
 	}
 
@@ -3000,8 +2877,8 @@ static int gpt_toggle_partition_flag(
 
 static int gpt_entry_cmp_start(const void *a, const void *b)
 {
-	struct gpt_entry *ae = (struct gpt_entry *) a,
-			 *be = (struct gpt_entry *) b;
+	const struct gpt_entry  *ae = (const struct gpt_entry *) a,
+				*be = (const struct gpt_entry *) b;
 	int au = gpt_entry_is_used(ae),
 	    bu = gpt_entry_is_used(be);
 
@@ -3140,13 +3017,11 @@ static const struct fdisk_field gpt_fields[] =
 /*
  * allocates GPT in-memory stuff
  */
-struct fdisk_label *fdisk_new_gpt_label(struct fdisk_context *cxt)
+struct fdisk_label *fdisk_new_gpt_label(struct fdisk_context *cxt __attribute__ ((__unused__)))
 {
 	struct fdisk_label *lb;
 	struct fdisk_gpt_label *gpt;
 
-	assert(cxt);
-
 	gpt = calloc(1, sizeof(*gpt));
 	if (!gpt)
 		return NULL;
