@@ -536,6 +536,7 @@ read_basicinfo(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 		else if (lookup(buf, "cpu family", &desc->family)) ;
 		else if (lookup(buf, "model", &desc->model)) ;
 		else if (lookup(buf, "CPU part", &desc->model)) ; /* ARM and aarch64 */
+		else if (lookup(buf, "cpu model", &desc->model)) ; /* mips */
 		else if (lookup(buf, "model name", &desc->modelname)) ;
 		else if (lookup(buf, "stepping", &desc->stepping)) ;
 		else if (lookup(buf, "CPU variant", &desc->stepping)) ; /* aarch64 */
@@ -545,6 +546,7 @@ read_basicinfo(struct lscpu_desc *desc, struct lscpu_modifier *mod)
 		else if (lookup(buf, "flags", &desc->flags)) ;		/* x86 */
 		else if (lookup(buf, "features", &desc->flags)) ;	/* s390 */
 		else if (lookup(buf, "Features", &desc->flags)) ;	/* aarch64 */
+		else if (lookup(buf, "ASEs implemented", &desc->flags)) ;	/* mips */
 		else if (lookup(buf, "type", &desc->flags)) ;		/* sparc64 */
 		else if (lookup(buf, "bogomips", &desc->bogomips)) ;
 		else if (lookup(buf, "BogoMIPS", &desc->bogomips)) ;	/* aarch64 */
@@ -1154,13 +1156,15 @@ read_topology(struct lscpu_desc *desc, int idx)
 		 */
 		desc->coremaps = xcalloc(desc->ncpuspos, sizeof(cpu_set_t *));
 		desc->socketmaps = xcalloc(desc->ncpuspos, sizeof(cpu_set_t *));
-		desc->coreids = xcalloc(desc->ncpuspos, sizeof(*desc->drawerids));
-		desc->socketids = xcalloc(desc->ncpuspos, sizeof(*desc->drawerids));
+		desc->coreids = xcalloc(desc->ncpuspos, sizeof(*desc->coreids));
+		desc->socketids = xcalloc(desc->ncpuspos, sizeof(*desc->socketids));
+
 		for (i = 0; i < desc->ncpuspos; i++)
 			desc->coreids[i] = desc->socketids[i] = -1;
+
 		if (book_siblings) {
 			desc->bookmaps = xcalloc(desc->ncpuspos, sizeof(cpu_set_t *));
-			desc->bookids = xcalloc(desc->ncpuspos, sizeof(*desc->drawerids));
+			desc->bookids = xcalloc(desc->ncpuspos, sizeof(*desc->bookids));
 			for (i = 0; i < desc->ncpuspos; i++)
 				desc->bookids[i] = -1;
 		}
@@ -1176,11 +1180,12 @@ read_topology(struct lscpu_desc *desc, int idx)
 	desc->coreids[idx] = coreid;
 	add_cpuset_to_array(desc->coremaps, &desc->ncores, thread_siblings);
 	desc->socketids[idx] = socketid;
-	if (book_siblings) {
+
+	if (book_siblings && desc->bookmaps && desc->bookids) {
 		add_cpuset_to_array(desc->bookmaps, &desc->nbooks, book_siblings);
 		desc->bookids[idx] = bookid;
 	}
-	if (drawer_siblings) {
+	if (drawer_siblings && desc->drawermaps && desc->drawerids) {
 		add_cpuset_to_array(desc->drawermaps, &desc->ndrawers, drawer_siblings);
 		desc->drawerids[idx] = drawerid;
 	}
@@ -1509,6 +1514,8 @@ get_cell_data(struct lscpu_desc *desc, int idx, int col,
 			snprintf(buf, bufsz, "%d", desc->idx2nodenum[i]);
 		break;
 	case COL_CPU_DRAWER:
+		if (!desc->drawerids || !desc->drawermaps)
+			break;
 		if (mod->physical) {
 			if (desc->drawerids[idx] == -1)
 				snprintf(buf, bufsz, "-");
@@ -1521,6 +1528,8 @@ get_cell_data(struct lscpu_desc *desc, int idx, int col,
 		}
 		break;
 	case COL_CPU_BOOK:
+		if (!desc->bookids || !desc->bookmaps)
+			break;
 		if (mod->physical) {
 			if (desc->bookids[idx] == -1)
 				snprintf(buf, bufsz, "-");
@@ -1843,6 +1852,7 @@ print_cpus_parsable(struct lscpu_desc *desc, int cols[], int ncols,
 			data = get_cell_data(desc, i, cols[c], mod,
 					     buf, sizeof(buf));
 			fputs(data && *data ? data : "", stdout);
+			*buf = '\0';
 		}
 		putchar('\n');
 	}
